
from ws2812 import *
from wsconfig import *

import random
import machine

def map_hue(value, v_min, v_max, h_min, h_max):
	value = (value - v_min) / (v_max - v_min)
	value = min(1, max(0, value))
	r, g, b = ydpg.hsv2rgb(round((h_min + value * (h_max - h_min))), 255, 127)
	return (b << 16) | (r << 8) | g

sea_temp_text = None
sea_temp_color = None
air_temp_text = None
air_temp_color = None

humidity_text = None
humidity_color = None

def temperature_extended(n2k, data, msgid, seq):
	global sea_temp_text, sea_temp_color
	global air_temp_text, air_temp_color
	raw = data[3] | (data[4] << 8) | (data[5] << 16)
	if raw <= 0xFFFFFC: # have valid temperature
		temp = raw * 0.001 - 273.15
		if data[2] == 0x0: # water temperature
			sea_temp_text = "{:.0f}&c".format(temp)
			sea_temp_color = map_hue(temp, -10, 40, 288, 0)
		elif data[2] == 0x1: # air temperature
			air_temp_text = "{:.0f}&c".format(temp)
			air_temp_color = map_hue(temp, -10, 40, 288, 0)
	return True

def humidity(n2k, data, msgid, seq):
	global humidity_text, humidity_color
	raw = data[3] | (data[4] << 8)
	if raw <= 0x7FFC: # have valid humidity
		hum = raw * 0.004
		
		humidity_text = "{:.0f}%".format(hum)
		humidity_color = map_hue(hum, 0, 100, 0, 360)
	return True

n2k.rxcallback(0, temperature_extended, 130316)
n2k.rxcallback(1, humidity, 130313)

def gen_time():
	global cfg
	dt = rtc.datetime()
	fmt = "{:02d}:{:02d}"
	if cfg.hold and (dt[6] % 2 == 0):
		fmt = "{:02d}`{:02d}" # '`' special character for 1 pixel space
	return fmt.format(dt[4], dt[5])

def bat_draw(ws, font, cfg, x, y):
	for I in range(6):
		xo = x + WS2812.WIDTH - 2 - I
		yo = y + WS2812.HEIGHT // 2
		ws.pixel(xo, yo + 1, 0xFFFFFF)
		ws.pixel(xo, yo - 2, 0xFFFFFF)
	for I in range(2):
		xo = x + WS2812.WIDTH - 2
		yo = y + WS2812.HEIGHT // 2 - 1 + I
		ws.pixel(xo, yo, 0xFFFFFF)
		ws.pixel(xo - 6, yo, 0xFFFFFF)
	
	volts = ydpg.busvoltage()
	percent = (volts - 10) / 4
	xo = x + WS2812.WIDTH - 3
	yo = y + WS2812.HEIGHT // 2
	
	if percent > 0:
		ws.pixel(xo, yo, 0x007F00)
		ws.pixel(xo, yo-1, 0x007F00)
		if percent > 0.20:
			xo -= 1
			ws.pixel(xo, yo, 0x006F3F)
			ws.pixel(xo, yo-1, 0x006F3F)
			if percent > 0.40:
				xo -= 1
				ws.pixel(xo, yo, 0x005F5F)
				ws.pixel(xo, yo-1, 0x005F5F)
				if percent > 0.60:
					xo -= 1
					ws.pixel(xo, yo, 0x003F6F)
					ws.pixel(xo, yo-1, 0x003F6F)
					if percent > 0.8:
						xo -= 1
						ws.pixel(xo, yo, 0x00007F)
						ws.pixel(xo, yo-1, 0x00007F)
	
	s = "{:.1f}v".format(volts)
	o = font.calcw(s)
	font.puts(o + x, y, s, 0xFFFFFF)
	
	return True

CONFIG = [
	{
		"type": TYPE_TICKER,
		#"fps": 50
	},{
		"anim": ANIM_HORIZONTAL,
		"ctext": gen_time,
	},{
		"anim": ANIM_VERTICAL,
		"draw": bat_draw
	},{
		"anim": ANIM_VERTICAL,
		
		"text": "AIR",
		"rtext": lambda: air_temp_text,
		"rcolor": lambda: air_temp_color,
	},{
		"anim": ANIM_VERTICAL,
		
		"text": "SEA",
		"rtext": lambda: sea_temp_text,
		"rcolor": lambda: sea_temp_color,
	},{
		"anim": ANIM_VERTICAL,
		
		"text": "RH",
		"rtext": lambda: humidity_text,
		"rcolor": lambda: humidity_color,
	},{
		"anim": ANIM_VERTICAL,
	}
]

uart_tx.deinit()
uart_rx.deinit()

pin = machine.Pin("rx")
ws = WS2812()
font = WSFont(ws)

cfg = WSConfig(pin, ws, font, CONFIG)
cfg.start()

n2k.rxcallback(0, None)
n2k.rxcallback(1, None)